@@ -2,6 +2,7 @@ import math
 from abc import ABC, abstractmethod
 from functools import lru_cache, partial
 from itertools import combinations
+from typing import Callable, List, Optional, Tuple
 
 import numpy as np
 import pandas as pd
@@ -51,11 +52,11 @@ def fast_object_mode(s):
 
 class Explainer(ABC):
     @abstractmethod
-    def explain(self, X):
+    def explain(self, x):
         raise NotImplementedError
 
 
-def update_columns(base, update):
+def update_columns(base: pd.DataFrame, update: pd.DataFrame) -> pd.DataFrame:
     """Update a base dataframe column-wise with a replacement dataframe
 
     Parameters
@@ -79,14 +80,14 @@ def update_columns(base, update):
     return combined.reindex(columns=base_columns)
 
 
-def reduce_func(func, X):
+def reduce_func(func: Callable[[pd.DataFrame], pd.Series], x: pd.DataFrame):
     """Reduce a function to accept an arbitrary subset of X as input
 
     Parameters
     ----------
     func : function
         Original function, must accept X as input
-    X : pd.DataFrame
+    x : pd.DataFrame
         Original dataframe, used for filling in missing columns
 
     Returns
@@ -95,60 +96,65 @@ def reduce_func(func, X):
         Reduced function that takes an arbitrary subset of X's columns as input
     """
 
-    def func_subset(X_subset, X, original_func):
-        X_updated = update_columns(base=X, update=X_subset)
+    def func_subset(x_subset, x_, original_func):
+        X_updated = update_columns(base=x_, update=x_subset)
         return original_func(X_updated)
 
-    reduced_func = partial(func_subset, X=X, original_func=func)
+    reduced_func = partial(func_subset, x_=x, original_func=func)
     return reduced_func
 
 
 class ShapExplainer(Explainer):
-    def __init__(self, func, X, profile_columns=None):
+    def __init__(
+        self,
+        func: Callable[[pd.DataFrame], pd.Series],
+        x: pd.DataFrame,
+        profile_columns: Optional[List[str]] = None,
+    ):
         """Create a Shap Explainer
 
         Parameters
         ----------
         func : function
             Function to be explained. Must map from X to array-like
-        X : pd.DataFrame
+        x : pd.DataFrame
             Background dataset for Shapley value calculation
         profile_columns : list, optional
             Calculate Shapley values only for a subset of columns, by default all columns will be included
         """
         if profile_columns is not None:
-            func = reduce_func(func, X=self._create_background_data(X))
-            X = X[profile_columns]
+            func = reduce_func(func, x=self._create_background_data(x))
+            x = x[profile_columns]
 
         self.func = func
-        self.X = self._create_background_data(X)
-        self.n_col = len(self.X.columns)
+        self.x = self._create_background_data(x)
+        self.n_col = len(self.x.columns)
 
     @staticmethod
-    def _create_background_data(X):
+    def _create_background_data(x) -> pd.DataFrame:
         """Create single-row background dataset"""
         background = {}
-        for col in X.columns:
-            if pd.api.types.is_numeric_dtype(X[col]):
-                background[col] = [X[col].median()]
-            elif pd.api.types.is_object_dtype(X[col]):
-                background[col] = [fast_object_mode(X[col])]
+        for col in x.columns:
+            if pd.api.types.is_numeric_dtype(x[col]):
+                background[col] = [x[col].median()]
+            elif pd.api.types.is_object_dtype(x[col]):
+                background[col] = [fast_object_mode(x[col])]
             else:
-                background[col] = [X[col].mode(dropna=False).values[0]]
-        X_background = pd.DataFrame(background, columns=X.columns).astype(X.dtypes)
-        return X_background
+                background[col] = [x[col].mode(dropna=False).values[0]]
+        x_background = pd.DataFrame(background, columns=x.columns).astype(x.dtypes)
+        return x_background
 
-    def explain(self, X):
+    def explain(self, x) -> Tuple[pd.Series, pd.DataFrame]:
         """Calculate Shapley explanations for a dataframe"""
         baselines = []
         explanations = []
-        for i in range(len(X)):  # loop over rows
-            row = X.iloc[[i]]
+        for i in range(len(x)):  # loop over rows
+            row = x.iloc[[i]]
             reference, explanation = self._explain_row(row)
             baselines.append(reference)
             explanations.append(explanation)
         baselines = pd.Series(baselines, name="Baseline")
-        explanations = pd.DataFrame(explanations, columns=X.columns)
+        explanations = pd.DataFrame(explanations, columns=x.columns)
         return baselines, explanations
 
     def _explain_row(self, row):
@@ -181,11 +187,11 @@ class ShapExplainer(Explainer):
         empty_coalition = {tuple([0] * self.n_col)}
         full_coalition = {tuple([1] * self.n_col)}
 
-        X_empty = self._map_features(empty_coalition, row)
-        X_full = self._map_features(full_coalition, row)
+        x_empty = self._map_features(empty_coalition, row)
+        x_full = self._map_features(full_coalition, row)
 
-        y_empty = np.float(self.func(X_empty))
-        y_full = np.float(self.func(X_full))
+        y_empty = np.float(self.func(x_empty))
+        y_full = np.float(self.func(x_full))
 
         if self.n_col == 1:
             # We have two constraints and two unknowns
@@ -196,8 +202,8 @@ class ShapExplainer(Explainer):
         else:
             # We actually need to sample coalitions and solve the WLS problem
             coalitions = self._sample_coalitions(self.n_col)
-            X = self._map_features(coalitions, row)
-            y = np.array(self.func(X))
+            x = self._map_features(coalitions, row)
+            y = np.array(self.func(x))
 
             weights = np.diag(self._coalition_weights(coalitions))
             indicators = np.array(list(coalitions), dtype=int)
@@ -245,14 +251,17 @@ class ShapExplainer(Explainer):
 
     def _map_features(self, coalitions, row):
         """Map coalition vectors to original feature space"""
-        X_mapped = row.copy().iloc[[0] * len(coalitions)]  # repeat row
+        x_mapped = row.copy().iloc[[0] * len(coalitions)]  # repeat row
 
         # for each row (coalition) replace values with value from  background dataset
         coalitions_array = np.array(list(coalitions), dtype=bool)
-        for i, col in enumerate(X_mapped.columns):
+
+        for i, col in enumerate(x_mapped.columns):
             inside = coalitions_array[:, i]
-            X_mapped[col].where(cond=inside, other=self.X[col].values, inplace=True)
-        return X_mapped
+            x_mapped[col].where(
+                cond=inside, other=self.x[col].values, inplace=True, try_cast=True
+            )
+        return x_mapped
 
     @staticmethod
     def _coalition_weights(coalitions):
